/****************************************************************************
** $Id: qseditor.cpp  beta4   edited Apr 28 16:41 $
**
** Implementation of the QSEditor class.
**
** Copyright (C) 2001-2003 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qseditor.h"
#include "qsinterpreter.h"
#include "qsproject.h"
#include "qsscript.h"

#include "../editor/qsacompletion.h"
#include "../editor/qsaeditor.h"
#include "../editor/qsaeditorinterface.h"

#include <qapplication.h>
#include <qobjectlist.h>
#include <qmetaobject.h>
#include <qsizepolicy.h>
#include <qtextedit.h>

class QSEditorPrivate
{
public:
    QSEditorPrivate() :
	source( 0 ),
	editor( 0 ),
	editorWidget( 0 )
    {
    }

    QSScript *source;
    QSAEditor *editor;
    QSAEditorInterface *eIface;
    QWidget *editorWidget;

    uint isUpdater : 1;
    uint modified : 1;
};

/*!
  \class QSEditor qseditor.h

  \brief The QSEditor class provides an enhanced editor for editing
  of Qt Script source code.

  For some applications the \e {Qt Scripter} is too complex. In this
  case it might be more appropriate to embed a simple editor into the
  application's UI.  QSEditor provides such a widget component, which
  offers in addition to normal editor functionality also Qt Script
  syntax highlighting, completion, function argument hints and
  auto-indentation.

  The text in the QSEditor can be set using setText() and retrieved
  using text().

  A QSEditor can be associated with a QSSctipt object using
  setScript() and the associated QSScript can be retrieved by using
  script().

  When text is edited in the QSEditor it is not directly reflected in
  the underlying script. To write the changes in the editor to the
  script, use commit(). To revert the changes in the editor to the
  text in the script, use revert().

  The QSEditor will automatically become the active editor when it
  gets focus. The active editor can be retrieved from QSProject.

  Look at the examples/simplescript, examples/scriptbutton and
  examples/console examples to see an example usage of the QSEditor
  class.
*/

/*! \fn void QSEditor::textChanged()
  This signal is emitted if the text of the editor has been changed
*/

/*! \overload
  Constructs an editor not linked to any project.

  The \a parent and \a name arguments are passed to the QWidget
  constructor.
*/

QSEditor::QSEditor( QWidget *parent, const char *name )
    : QWidget( parent, name )
{
    d = new QSEditorPrivate;
    init();
}

/*! \internal
 */

void QSEditor::init()
{
    d->eIface = new QSAEditorInterface();
    QWidget *editor = d->eIface->editor( FALSE, this );
    Q_ASSERT( editor->inherits( "QSAEditor" ) );
    d->editor = (QSAEditor*) editor;
    d->editorWidget = editor->parentWidget();

    setFocusPolicy( WheelFocus );
    setFocusProxy( editor );
    setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    connect( d->editor, SIGNAL( textChanged() ),
	     this, SIGNAL( textChanged() ) );
    connect( this, SIGNAL( textChanged() ),
	     this, SLOT( modifyEditor() ) );
    editor->setGeometry( 0, 0, width(), height() );
    editor->show();

    d->isUpdater = FALSE;
    d->modified = FALSE;

    QObjectList *l = queryList( "QWidget" );
    for ( QObject *o = l->first(); o; o = l->next() )
	o->installEventFilter( this );
    delete l;
}


/*!
  Destructs the editor.
*/
QSEditor::~QSEditor()
{
    delete d;
}


/*!
  Returns the project this editor is associated; otherwise 0;
*/
QSProject *QSEditor::project() const
{
    return d->source ? d->source->project() : 0 ;
}


/*!
  Sets the editor contents to \a text.
*/
void QSEditor::setText( const QString &text )
{
    d->editor->setText( text );
}


/*!
  Returns the text of the editor
*/
QString QSEditor::text() const
{
    return d->editor->text();
}


/*!
  Updates the script with the text in the editor.

  When the text in the editor is changed, one must commit
  the changes before the changes are reflected in the script.
*/
void QSEditor::commit()
{
    if( d->source ) {
	d->isUpdater = TRUE; // Avoid recursion...
	d->source->setCode( text() );
	d->isUpdater = FALSE;
	d->modified = FALSE;
    }
}


/*!
  Reverts the contents in the editor back to the contents
  of the underlying script.

  When the text in the editor is changed and one wishes to
  revert it to the state of the underlying script, revert
  is used.
*/
void QSEditor::revert()
{
    setText( d->source->code() );
    d->modified = FALSE;
}


/*!
  Returns TRUE if the text in the editor is different from the
  text in the underlying script. This is the case whenever the
  user types input and up til the user does either commit or
  revert.
*/
bool QSEditor::isModified() const
{
    return d->modified;
}


/*!
  Tries to activate this editor. Returns TRUE if it succeeds,
  otherwise FALSE.

  See the class description for the concept of active editors.
*/
bool QSEditor::activate()
{
    QSProject *proj = project();
    if( proj )
	proj->evaluate();
    QSEditor *ed = ( proj ? proj->activeEditor() : 0 );
    if ( ed )
	ed->release();
    active = TRUE;
    return TRUE;
}


/*!
  Releases (deactivates) this editor.

  See the class description for the concept of active editors.
*/
void QSEditor::release()
{
    active = FALSE;
}


/*!
  \fn bool QSEditor::isActive() const
  Returns whether this editor is active or not.

  See the class description for the concept of active editors.
*/


/*!
  Returns whether undo is available
*/
bool QSEditor::isUndoAvailable() const
{
    return d->editor->isUndoAvailable();
}


/*!
  Returns whether redo is available
*/
bool QSEditor::isRedoAvailable() const
{
    return d->editor->isRedoAvailable();
}

/*!
  Undoes the last editor operation
*/
void QSEditor::undo()
{
    d->editor->undo();
}


/*!
  Redoes the last editor operation
*/
void QSEditor::redo()
{
    d->editor->redo();
}


/*!
  Cuts the selected text to the clipboard
*/
void QSEditor::cut()
{
    d->editor->cut();
}


/*!
  Copies the selected text to the clipboard
*/
void QSEditor::copy()
{
    d->editor->copy();
}


/*!
  Pastes text from the clipboard into the editor
*/
void QSEditor::paste()
{
    d->editor->paste();
}


/*!
  Selects all the text in the editor
*/
void QSEditor::selectAll()
{
    d->editor->selectAll();
}


/*!
  Finds the expression \a expr in the editor and selects the
  result. If \a cs is TRUE the search is done case sensitive. If \a wo
  is TRUE, only whole words are searched. If \a forward is TRUE, the
  search is done formward, otherwise backwards. If \a startAtCursor is
  TRUE, the search starts at the current cursor position, otherwise it
  starts at the beginning of the document.

  Returns TRUE, of \a expr was found, FALSE otherwise.
*/
bool QSEditor::find( const QString &expr, bool cs, bool wo,
		     bool forward, bool startAtCursor )
{
    return d->eIface->find( expr, cs, wo, forward, startAtCursor );
}


/*!
  Replaces the expression \a find in the editor with \a replace. If
  \a cs is TRUE the search is done case sensitive. If \a wo is TRUE,
  only whole words are searched. If \a forward is TRUE, the search is
  done formward, otherwise backwards. If \a startAtCursor is TRUE, the
  search starts at the current cursor position, otherwise it starts at
  the beginning of the document. If \a replaceAll is TRUE, all found
  occurances of \a find are replaced by \a replace. Otherwise only the
  first occurance is replaced.
*/
bool QSEditor::replace( const QString &find, const QString &replace, bool cs, bool wo,
			bool forward, bool startAtCursor, bool replaceAll )
{
    return d->eIface->replace( find, replace, cs, wo, forward, startAtCursor, replaceAll );
}


/*!
  Moves the cursor to the line \a line in the editor
*/
void QSEditor::gotoLine( int line )
{
    d->eIface->gotoLine( line - 1 );
}


/*!
  Indents the current line or selection
*/
void QSEditor::indent()
{
    d->eIface->indent();
}


/*!
  Re-reads settings for syntax highlighting, indentation, etc. The
  settings are read from the QSettings path
  \c /Trolltech/QSAScriptEditor/

  Following settings are read for syntax highlighting:

  \list
  \i \c /Comment
  \i \c /Number
  \i \c /String
  \i \c /Type
  \i \c /Label
  \i \c /Standard
  \endlist

  For each of those following settings are read:

  \list
  \i \c /family (string)
  \i \c /size (number)
  \i \c /bold (bool)
  \i \c /italic (bool)
  \i \c /underline (bool)
  \i \c /red (number)
  \i \c /green (number)
  \i \c /blue (number)
  \endlist

  So, the font family for strings is e.g. saved in
  \c /Trolltech/QuickScriptEditor/String/family.

  Following settings for indentation are read:

  \list
  \i \c /indentAutoIndent (bool) - whether auto indentation should be done
  \i \c /indentTabSize (number) - the tab size for indentation
  \i \c /indentIndentSize (number) - the indentation width
  \i \c /indentKeepTabs (bool) - whether to keep tabs or to replace them with spaces
  \endlist

  Code complection can be switched on or off via \c /completion,
  wordwrapping via \c /wordWrap and paren matching via \c /parenMatching
*/
void QSEditor::readSettings()
{
    d->eIface->readSettings();
}


/*!
  Highlights the line \a line. This can be used e.g. to show and
  error or warning
*/
void QSEditor::setErrorMark( int line )
{
    d->eIface->setError( line - 1 );
}


/*!
  Removes all the error marks for this editor.
 */
void QSEditor::removeErrorMark()
{
    d->eIface->clearError();
}

/*!
  \reimp
*/
QSize QSEditor::sizeHint() const
{
    return d->editorWidget ? d->editorWidget->sizeHint() : QSize();
}


/*!
  \reimp
*/
QSize QSEditor::minimumSizeHint() const
{
    return d->editorWidget ? d->editorWidget->minimumSizeHint() : QSize();
}


/*!
  \reimp
*/
void QSEditor::resizeEvent( QResizeEvent *e )
{
    if( d->editorWidget ) {
	QWidget::resizeEvent( e );
	d->editorWidget->setGeometry( 0, 0, width(), height() );
    }
}


/*!
  \reimp
*/
bool QSEditor::eventFilter( QObject *o, QEvent *e )
{
    if ( e->type() == QEvent::FocusIn )
	activate();
    return QWidget::eventFilter( o, e );
}


/*!
  Returns the pointer to the real editor widget, which is a
  QTextEdit. This might be 0 if an error occured while initializing
  the editor.
*/
QTextEdit *QSEditor::textEdit() const
{
    return d->editor;
}


/*!
  Returns the script that this editor is editing if any; otherwise 0.
 */
QSScript *QSEditor::script() const
{
    Q_ASSERT( d->source );
    return d->source;
}


/*!
  Sets the current script of this editor to be \a script.


 */
bool QSEditor::setScript( QSScript *script )
{
    if( !script ) {
	return FALSE;
    }

    if( d->source && d->source->project() != script->project() ) {
#if defined( QT_RANGE_CHECK )
	qWarning( "QSEditor::setScript(), new script cannot belongs to different project" );
#endif
	return FALSE;
    }

    if( d->source )
	disconnect( d->source, SIGNAL( codeChanged() ),
		    this, SLOT( scriptChanged() ) );

    script->project()->registerEditor( this );

    d->source = script;
    ( (QSACompletion*) d->editor->completionManager() )->setContext( script->context() );
    setText( script->code() );
    d->editor->setProject( script->project() );

    connect( script, SIGNAL( codeChanged() ),
	     this, SLOT( scriptChanged() ) );

    d->modified = FALSE;

    return TRUE;
}


void QSEditor::scriptChanged()
{
    Q_ASSERT( d->source );
    if( !d->isUpdater )
	setText( d->source->code() );
}

void QSEditor::modifyEditor()
{
    d->modified = TRUE;
}
