/****************************************************************************
** $Id: qsobjectfactory.cpp  beta4   edited Apr 25 22:31 $
**
** Copyright (C) 2001-2003 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsobjectfactory.h"
#include "qsinterfacefactory_p.h"
#include "qsinterpreter.h"
#include "quickinterpreter.h"
#include "qsenv.h"

class QSObjectFactoryPrivate
{
public:
    QSInterpreter *ip;
    QMap<QString,QObject> statics;
};

/*!
  \class QSObjectFactory qsobjectfactory.h

  \brief The QSObjectFactory class provides a mechanism for Qt Script
  programs to create C++ QObjects.

  If we want script programmers to create their own C++ QObjects we
  can either provide a QObject subclass that has a slot which is a
  factory function that returns QObjects, or we can subclass
  QSObjectFactory and reimplement the create() and classes()
  functions.

  The classes() function is called by the scripting engine to see
  which classes can be construced by the script programmer. And the
  create() function is called by the scripting engine to create the
  instance, i.e. when the user writes something like this:
  \code
    var x = new SomeCppObject( arg1, arg2 ); // Qt Script
  \endcode

  The staticClasses() function is called by the scripting engine
  to see which classes provide static functions and variables.
  A QObject instance is then later created using createStatic()
  to provide the interpreter of the static representation.

  A single QSObjectFactory subclass may be used to provide any number of
  creatable QObject classes. To make these classes known to the
  scripting engine, create an instance of the subclass with a pointer
  to the interpreter as an argument.

  A QSObjectFactory becomes active when it is added to a QSInterpreter
  using the function QSInterpreter::addObjectFactory(). An object factory
  can only be added to one QSInterpreter at a time.
*/


/*! Constructor.
  Creates the object factory. To make the object factory available to
  and interpreter, use the function QSInterpreter::addObjectFactory().
*/

QSObjectFactory::QSObjectFactory()
{
    d = new QSObjectFactoryPrivate;
    d->ip = 0;
}


/*!
  \internal
*/
QSObjectFactory::~QSObjectFactory()
{
    delete d;
}

/*!
    Informs the interpreter than an error occured. The error is
    treated like a normal Qt Script error. The error message is passed
    in \a message.
*/

void QSObjectFactory::throwError( const QString &message )
{
    if( !d->ip ) {
#if defined ( QT_CHECK_RANGE )
	qWarning( "QSObjectFactory::throwError(), no interpreter" );
#endif
	return;
    }
    d->ip->throwError( message );
}


/*!
  Returns the interpreter that this object factory is creating
  objects for.
*/
QSInterpreter *QSObjectFactory::interpreter() const
{
    return d->ip;
}

/*!
  \internal
*/
void QSObjectFactory::setInterpreter( QSInterpreter *i )
{
    if( d->ip ) {
#if defined( QT_CHECK_STATE )
	qWarning( "QSObjectFactory::setInterpreter(), already has an interpreter" );
#endif
	return;
    }
    d->ip = i;
}


/*!
  This virtual function should be reimplemented in your
  QSObjectFactory subclass when you want to instantiate objects from
  script. It should return a list of the names of all the classes that
  your QSObjectFactory is capable of instantiating.

  This function is used by the scripting engine to find out which
  classes this QSObjectFactory can instantiate.
*/
QStringList QSObjectFactory::classes() const
{
    return QStringList();
}


/*!
  This virtual function should be reimplemented in your
  QSObjectFactory subclass when you want to instantiate objects from
  script. The subclass can be used to create any number of different
  classes; the name of the required class is passed in the \a
  className argument, and the arguments to be passed to that class's
  constructor are passed in the \a arguments list. See QSArgument for
  further information on the arguments. \a context os the script
  QObject context in which the class has been instantiated, or 0 if
  the class has not been instantiated in a QObject context.

  Only QObject subclasses may be created in this way. This
  function returns an instance of the requested class.

  This function is called by the scripting engine, e.g. when it
  encounters code like this:
  \code
    var x = new ACppObject( arg1, arg2 ); // Qt Script
  \endcode

  The classes that a particular QSObjectFactory instance is capable of
  instantiating is returned by classes().

  If the argumenst are invalid or any other invalid operation happens,
  you can use throwError() to issue a Qt Script error.
*/
QObject *QSObjectFactory::create( const QString &, const QSArgumentList &,
				  QObject * )
{
    return 0;
}


/*!
  This virtual function should be reimplemented in your
  QSObjectFactory subclass when you want to provide classes with
  static contents. It should return a list of the names of all the
  classes that your QSObjectFactory will provide static contents for.

  This function is used by the scripting engine to find out which
  classes this QSObjectFactory can instantiate.
 */
QStringList QSObjectFactory::staticClasses() const
{
    return QStringList();
}


/*!
  This virtual function should be reimplemented in your
  QSObjectFactory subclass when you want to provide classes with
  static contents. The class name is specified by \a className. Unlike
  instances, static classes will always be located in the global
  scope.

  This function will return a QObject which slots and properties will
  will exposed to the interpreter as static variables and functions.

  This function is called by the scripting engine during initialization
  and can be used in code like this:
  \code
    MessageBox.information( "This is a static function" );
  \endcode

  The classes that a particular QSObjectFactory instance is capable of
  providing static contents for is returned by staticClasses().
*/
QObject *QSObjectFactory::createStatic( const QString & )
{
    return 0;
}
